LIBRARY amd_ags_x64_orig
EXPORTS
agsInitialize
agsDeInitialize
agsCheckDriverVersion
agsDriverExtensionsDX12_CreateDevice
agsDriverExtensionsDX12_DestroyDevice
agsDriverExtensionsDX12_SetPrimitiveTopology
agsDriverExtensionsDX12_CreateGraphicsPipelineState
agsGetVersionNumber
agsSetDisplayMode
agsDriverExtensionsDX12_PushMarker
agsDriverExtensionsDX12_PopMarker
agsDriverExtensionsDX12_SetMarker
agsDriverExtensionsDX12_CreateComputePipelineState
agsDriverExtensionsDX12_CreatePipelineState
agsDriverExtensionsDX11_CreateDevice
agsDriverExtensionsDX11_DestroyDevice
agsDriverExtensionsDX11_WriteBreadcrumb
agsDriverExtensionsDX11_IASetPrimitiveTopology
agsDriverExtensionsDX11_BeginUAVOverlap
agsDriverExtensionsDX11_EndUAVOverlap
agsDriverExtensionsDX11_SetDepthBounds
agsDriverExtensionsDX11_MultiDrawInstancedIndirect
agsDriverExtensionsDX11_MultiDrawIndexedInstancedIndirect
agsDriverExtensionsDX11_MultiDrawInstancedIndirectCountIndirect
agsDriverExtensionsDX11_MultiDrawIndexedInstancedIndirectCountIndirect
agsDriverExtensionsDX11_SetMaxAsyncCompileThreadCount
agsDriverExtensionsDX11_NumPendingAsyncCompileJobs
agsDriverExtensionsDX11_SetDiskShaderCacheEnabled
agsDriverExtensionsDX11_SetViewBroadcastMasks
agsDriverExtensionsDX11_GetMaxClipRects
agsDriverExtensionsDX11_SetClipRects
agsDriverExtensionsDX11_CreateBuffer
agsDriverExtensionsDX11_CreateTexture1D
agsDriverExtensionsDX11_CreateTexture2D
agsDriverExtensionsDX11_CreateTexture3D
agsDriverExtensionsDX11_NotifyResourceEndWrites
agsDriverExtensionsDX11_NotifyResourceBeginAllAccess
agsDriverExtensionsDX11_NotifyResourceEndAllAccess
