LIBRARY amd_ags_x64_capture
EXPORTS
agsInitialize=gfxrecon_agsInitialize @1
agsDeInitialize=gfxrecon_agsDeInitialize @2
agsCheckDriverVersion=gfxrecon_agsCheckDriverVersion @3
agsDriverExtensionsDX12_CreateDevice=gfxrecon_agsDriverExtensionsDX12_CreateDevice @4
agsDriverExtensionsDX12_DestroyDevice=gfxrecon_agsDriverExtensionsDX12_DestroyDevice @5
agsGetVersionNumber=gfxrecon_agsGetVersionNumber @6
agsSetDisplayMode=gfxrecon_agsSetDisplayMode @7
agsDriverExtensionsDX12_PushMarker=gfxrecon_agsDriverExtensionsDX12_PushMarker @8
agsDriverExtensionsDX12_PopMarker=gfxrecon_agsDriverExtensionsDX12_PopMarker @9
agsDriverExtensionsDX12_SetMarker=gfxrecon_agsDriverExtensionsDX12_SetMarker @10
agsDriverExtensionsDX11_CreateDevice=amd_ags_x64_orig.agsDriverExtensionsDX11_CreateDevice @11
agsDriverExtensionsDX11_DestroyDevice=amd_ags_x64_orig.agsDriverExtensionsDX11_DestroyDevice @12
agsDriverExtensionsDX11_WriteBreadcrumb=amd_ags_x64_orig.agsDriverExtensionsDX11_WriteBreadcrumb @13
agsDriverExtensionsDX11_IASetPrimitiveTopology=amd_ags_x64_orig.agsDriverExtensionsDX11_IASetPrimitiveTopology @14
agsDriverExtensionsDX11_BeginUAVOverlap=amd_ags_x64_orig.agsDriverExtensionsDX11_BeginUAVOverlap @15
agsDriverExtensionsDX11_EndUAVOverlap=amd_ags_x64_orig.agsDriverExtensionsDX11_EndUAVOverlap @16
agsDriverExtensionsDX11_SetDepthBounds=amd_ags_x64_orig.agsDriverExtensionsDX11_SetDepthBounds @17
agsDriverExtensionsDX11_MultiDrawInstancedIndirect=amd_ags_x64_orig.agsDriverExtensionsDX11_MultiDrawInstancedIndirect @18
agsDriverExtensionsDX11_MultiDrawIndexedInstancedIndirect=amd_ags_x64_orig.agsDriverExtensionsDX11_MultiDrawIndexedInstancedIndirect @19
agsDriverExtensionsDX11_MultiDrawInstancedIndirectCountIndirect=amd_ags_x64_orig.agsDriverExtensionsDX11_MultiDrawInstancedIndirectCountIndirect @20
agsDriverExtensionsDX11_MultiDrawIndexedInstancedIndirectCountIndirect=amd_ags_x64_orig.agsDriverExtensionsDX11_MultiDrawIndexedInstancedIndirectCountIndirect @21
agsDriverExtensionsDX11_SetMaxAsyncCompileThreadCount=amd_ags_x64_orig.agsDriverExtensionsDX11_SetMaxAsyncCompileThreadCount @22
agsDriverExtensionsDX11_NumPendingAsyncCompileJobs=amd_ags_x64_orig.agsDriverExtensionsDX11_NumPendingAsyncCompileJobs @23
agsDriverExtensionsDX11_SetDiskShaderCacheEnabled=amd_ags_x64_orig.agsDriverExtensionsDX11_SetDiskShaderCacheEnabled @24
agsDriverExtensionsDX11_SetViewBroadcastMasks=amd_ags_x64_orig.agsDriverExtensionsDX11_SetViewBroadcastMasks @25
agsDriverExtensionsDX11_GetMaxClipRects=amd_ags_x64_orig.agsDriverExtensionsDX11_GetMaxClipRects @26
agsDriverExtensionsDX11_SetClipRects=amd_ags_x64_orig.agsDriverExtensionsDX11_SetClipRects @27
agsDriverExtensionsDX11_CreateBuffer=amd_ags_x64_orig.agsDriverExtensionsDX11_CreateBuffer @28
agsDriverExtensionsDX11_CreateTexture1D=amd_ags_x64_orig.agsDriverExtensionsDX11_CreateTexture1D @29
agsDriverExtensionsDX11_CreateTexture2D=amd_ags_x64_orig.agsDriverExtensionsDX11_CreateTexture2D @30
agsDriverExtensionsDX11_CreateTexture3D=amd_ags_x64_orig.agsDriverExtensionsDX11_CreateTexture3D @31
agsDriverExtensionsDX11_NotifyResourceEndWrites=amd_ags_x64_orig.agsDriverExtensionsDX11_NotifyResourceEndWrites @32
agsDriverExtensionsDX11_NotifyResourceBeginAllAccess=amd_ags_x64_orig.agsDriverExtensionsDX11_NotifyResourceBeginAllAccess @33
agsDriverExtensionsDX11_NotifyResourceEndAllAccess=amd_ags_x64_orig.agsDriverExtensionsDX11_NotifyResourceEndAllAccess @34
agsInit=gfxrecon_agsInit @35
agsDeInit=gfxrecon_agsDeInit @36
