LIBRARY d3d12_ms
EXPORTS
D3D12CoreCreateLayeredDevice @104
D3D12CoreGetLayeredDeviceSize @105
D3D12CoreRegisterLayers @106
D3D12CreateDevice @101
D3D12CreateRootSignatureDeserializer @107
D3D12CreateVersionedRootSignatureDeserializer @108
D3D12DeviceRemovedExtendedData @109
D3D12EnableExperimentalFeatures @110
D3D12GetDebugInterface @102
D3D12PIXEventsReplaceBlock @111
D3D12PIXGetThreadInfo @112
D3D12PIXNotifyWakeFromFenceSignal @113
D3D12PIXReportCounter @114
D3D12SerializeRootSignature @115
D3D12SerializeVersionedRootSignature @116
D3D12GetInterface @117
GetBehaviorValue @100
SetAppCompatStringPointer @103
